/*
 =============================================================
 PROJECT : FULTRA Gulpfile.js
 =============================================================
 */

"use strict";

//  Libs & Configs
var path = require('path');
var pkg = require('./package.json');
var rimraf = require('rimraf');
var gulp = require('gulp');
var gUtil = require('gulp-util');
var gProcessHtml = require('gulp-processhtml');
var gSass = require('gulp-sass');
var gSourceMaps = require('gulp-sourcemaps');
var gConcat = require('gulp-concat');
var browserSync = require('browser-sync').create();
var taskListing = require('gulp-task-listing');
var router = require('front-router');
var gHtml2Js = require('gulp-ng-html2js');
var gAutoPrefixer = require('gulp-autoprefixer');
var runSequence = require('run-sequence');

var gProcessHtmlSettings = {
    "process": true,
    "environment": (gUtil.env.env ? gUtil.env.env : "dist"),
    "data": {
        "title": "Documentation",
        "message": "This is Dist environment",
        "comment": "These comments will all be removed in the release environment"
    }
};

// General Tasks
gulp.task('default', ['help'], function(){
    var notify = require('gulp-notify');
    return notify("Hello Gulp!");
});
gulp.task('info', function(){
    gUtil.log(gUtil.colors.cyan('Project Fultra'));
    gUtil.log(gUtil.colors.cyan('----------------------------------------------'));
    gUtil.log(gUtil.colors.cyan('Version: '), pkg.version);
    gUtil.log(gUtil.colors.cyan('----------------------------------------------'));
});
gulp.task('help', taskListing);

// Clean Tasks
gulp.task('clean', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('clean:build', function(callback){
    return rimraf('./build',callback)
});

// Launch Tasks
gulp.task('launch', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('launch:docs', ['build'], function () {
    
    // When the server is running let the user know
    browserSync.emitter.on("init", function () {
        gUtil.log("Browsersync is running!");
    });
    
    // Initiate the server
    browserSync.init({
        server: {
            baseDir: "./build",
            directory: false,
            index: "index.html",
            //https: true,
            ghostMode: {
                clicks: true,
                forms: true,
                scroll: false
            }
        },
        logLevel: (gUtil.env.debug ? gUtil.env.debug : "silent"), //debug, info or silent
        logPrefix: "Host",
        logConnections: true,
        logFileChanges: true,
        notify: true,
        tunnel: false, //can replace with name
        online: true, //Will assume we are offline no tunnel,
        open: true // open, false, local, external, ui, tunnel
    });

    // Run watch tasks to watch elements and run corresponding build task
    
    //Watch scss
    gulp.watch(['./docs/assets/scss/*.scss','./scss/**'], ['build:css']);
    
    gulp.watch(['./vendor/foundation-apps/js/angular/components/**/*.html'], ['build:templateCache', 'build:js']);
    
    gulp.watch(['./docs/*.html','./docs/**/*.html'], ['build:html', 'build:js']);
    
    gulp.watch(['./docs/assets/js/*.js','./js/*.js'],['build:js']);
    
    gulp.watch('./build/**').on('change', browserSync.reload);
});

// Build Tasks
gulp.task('build', ['clean:build'],function(callback){
    runSequence('build:html','build:css','build:fonts','build:templateCache','build:js',callback)
});
gulp.task('build:html', function () {
    return gulp.src(
        [
            './docs/*.html',
            './docs/**/*.html'
        ]
    )
        .pipe(router({
            path: './docs/assets/js/routes.js',
            root: './docs/'
        }))
        .pipe(gProcessHtml(gProcessHtmlSettings))
        .pipe(gulp.dest('./build'));
});
gulp.task('build:css', function(){
    return gulp.src('./docs/assets/scss/*.scss')
        .pipe(gSourceMaps.init())
        .pipe(gSass({
            includePaths: [
                './scss/',
                './vendor/'
            ],
            outputStyle: "nested",
            errLogToConsole: true
        }).on('error', gSass.logError))
        .pipe(gAutoPrefixer({
            browsers: ['last 2 versions'],
            cascade: false
        }))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./build/assets/css'));
});
gulp.task('build:settings',function(cb){
//    var octophant = require('octophant');
//
//    return octophant(['./libs/src/scss/', './libs/vendor/foundation-apps/scss/'], {
//                title: 'Fultra for Sites Settings',
//                output: './libs/src/scss/_settings.scss',
//                groups: {
//                        'grid': 'The Grid',
//                        'off-canvas': 'Off-canvas',
//                        'typography-base': 'Base Typography'
//                      },
//                      sort: [
//                        'global',
//                        'breakpoints',
//                        'grid',
//                        'typography-base',
//                        'typography-helpers'
//                      ],
//                      imports: ['util/util'],
//                      _foundationShim: true
//              }, cb());
});
gulp.task('build:templateCache', function(){
    return gulp.src('./vendor/foundation-apps/js/angular/components/**/*.html')
        .pipe(gHtml2Js({
            moduleName: "foundation.templates",
            prefix: "components/"
        }))
        .pipe(gConcat('foundation.templates.js'))
        .pipe(gulp.dest('./docs/assets/js'));
});
gulp.task('build:js', function(){
    return gulp.src(
        [
           // './libs/vendor/jquery/dist/jquery.js',
           // Angular and deps
            './vendor/angular/angular.js',
            './vendor/angular-animate/angular-animate.js',
            './vendor/angular-ui-router/release/angular-ui-router.js',
            // Any Jquery elements
            './vendor/fastclick/lib/fastclick.js',
            './vendor/hammerjs/hammer.js',
            './vendor/tether/dist/js/tether.js',
            // Foundation Apps Components & Services
            './vendor/foundation-apps/js/angular/services/*.js',
            './vendor/foundation-apps/js/angular/components/**/*.js',
            './vendor/foundation-apps/js/angular/vendor/**/*.js',
            './vendor/foundation-apps/js/vendor/**/*.js',
            // Bring it all together with the fultra namespace
            './js/fultra.js',
            // All of the foundation html templates into a js cache module
            './docs/assets/js/foundation.templates.js',
            // Our generated routes
            './docs/assets/js/routes.js',
            // And finally our application specific scripts
            './docs/assets/js/app.js'
        ]
    )
        .pipe(gSourceMaps.init())
        .pipe(gConcat('app.js'))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./build/assets/js/'))
});
gulp.task('build:fonts', function(){
    gulp.src('./fonts/**')
        .pipe(gulp.dest('./build/assets/fonts/fultra/'));
    gulp.src('./vendor/foundation-apps/iconic/**')
        .pipe(gulp.dest('./build/assets/fonts/iconic/'));
    return gulp.src('./vendor/font-awesome/fonts/**')
        .pipe(gulp.dest('./build/assets/fonts/font-awesome/'));
});

// Release Tasks
gulp.task('release', function(callback){
    runSequence('release:css','release:fonts','release:templateCache','release:js',callback)
});
gulp.task('release:css', function(){
    return gulp.src('./scss/*.scss')
        .pipe(gSourceMaps.init())
        .pipe(gSass({
            includePaths: [
                './scss/',
                './vendor/'
            ],
            outputStyle: "nested",
            errLogToConsole: true
        }).on('error', gSass.logError))
        .pipe(gAutoPrefixer({
            browsers: ['last 2 versions'],
            cascade: false
        }))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./dist/css'));
});
gulp.task('release:templateCache', function(){
    return gulp.src('./vendor/foundation-apps/js/angular/components/**/*.html')
        .pipe(gHtml2Js({
            moduleName: "foundation.templates",
            prefix: "components/"
        }))
        .pipe(gConcat('foundation.templates.js'))
        .pipe(gulp.dest('./dist/js'));
});
gulp.task('release:js', function(){
    return gulp.src(
        [
            // './libs/vendor/jquery/dist/jquery.js',
            // Angular and deps
            './vendor/angular/angular.js',
            './vendor/angular-animate/angular-animate.js',
            './vendor/angular-ui-router/release/angular-ui-router.js',
            // Any Jquery elements
            './vendor/fastclick/lib/fastclick.js',
            './vendor/hammerjs/hammer.js',
            './vendor/tether/dist/js/tether.js',
            // Foundation Apps Components & Services
            './vendor/foundation-apps/js/angular/services/*.js',
            './vendor/foundation-apps/js/angular/components/**/*.js',
            './vendor/foundation-apps/js/angular/vendor/**/*.js',
            './vendor/foundation-apps/js/vendor/**/*.js',
            // Bring it all together with the fultra namespace
            './js/fultra.js'
        ]
        )
        .pipe(gSourceMaps.init())
        .pipe(gConcat('fultra.js'))
        .pipe(gSourceMaps.write())
        .pipe(gulp.dest('./dist/js/'))
});
gulp.task('release:fonts', function(){
    gulp.src('./vendor/foundation-apps/iconic/**')
        .pipe(gulp.dest('./dist/fonts/iconic/'));
    return gulp.src('./vendor/font-awesome/fonts/**')
        .pipe(gulp.dest('./dist/fonts/font-awesome/'));
});

// Lint Tasks
gulp.task('lint', function(callback){
    runSequence('lint:scss','lint:css','lint:js',callback);
});
gulp.task('lint:scss', function(){
    var gScssLint = require('gulp-scss-lint');

   return gulp.src([
       './libs/src/scss/*.scss',
       './docs/src/assets/scss/*.scss'
   ])
        .pipe(gScssLint());
});
gulp.task('lint:css', function(){
    var gCssLint = require('gulp-csslint');
    return gulp.src('./docs/release/assets/css/*.css')
        .pipe(gCssLint())
        .pipe(gCssLint.formatter(require('csslint-stylish')))
});
gulp.task('lint:js', function(){
    var gJsHint = require('gulp-jshint');
    return gulp.src([
        './libs/src/js/*.js',
        './docs/src/assets/js/app.js'
    ])
        .pipe(gJsHint())
        .pipe(gJsHint.reporter('jshint-stylish'))
        //.pipe(gJsHint.reporter('jshint-stylish', {beep: true}))
       // .pipe(gJsHint.reporter('fail'))
});

// Testing Tasks
gulp.task('test', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('test:css', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});
gulp.task('test:js', function(){gUtil.log(gUtil.colors.yellow('Task Empty!!'))});

